@@ -20,28 +20,23 @@ Don't pollute the B<< namespace >> of Net::IPAM::Block
 =cut
 
 # only masks without a gap, representable as CIDR, are allowed
-my %valid_masks = map { pack( 'N', ( 0xffff_ffff >> $_ ) << $_ ) => $_ } 0 .. 32;
-
+#
 # 11111111_11111111_11111111_11111111
 # 11111111_11111111_11111111_11111110
 # 11111111_11111111_11111111_11111100
-# 11111111_11111111_11111111_11111000
-# 11111111_11111111_11111111_11110000
-# 11111111_11111111_11111111_11100000
-# 11111111_11111111_11111111_11000000
 # ...
-# 11111100_00000000_00000000_00000000
-# 11111000_00000000_00000000_00000000
-# 11110000_00000000_00000000_00000000
 # 11100000_00000000_00000000_00000000
 # 11000000_00000000_00000000_00000000
 # 10000000_00000000_00000000_00000000
+my %valid_masks = map { pack( 'N', ( 0xffff_ffff >> $_ ) << $_ ) => $_ } 0 .. 31;
+
 # 00000000_00000000_00000000_00000000
+# extra treatment for 32bit, still support 32 bit perl, thanks to the smoke testers!
+$valid_masks{ pack( 'N', 0x0000_0000 ) } = 32;
 
 # 1.2.3.4/255.0.0.0  => {base: 1.0.0.0, last: 1.255.255.255, mask: 255.0.0.0}
 # 1.2.3.4/24         => {base: 1.2.3.0, last: 1.2.3.255,     mask: 255.255.255.0}
 # fe80::1/124 =>        {base: fe80::,  last: fe80::f,       mask: ffff:ffff:ffff:ffff:ffff:ffff:ffff:fff0}
-
 sub _fromMask {
   my ( $self, $addr, $idx ) = @_;
 
